Type: Package
Package: officer
Title: Manipulation of Microsoft Word and PowerPoint Documents
Version: 0.7.5
Authors@R: c(
    person("David", "Gohel", , "david.gohel@ardata.fr", role = c("aut", "cre")),
    person("Stefan", "Moog", , "moogs@gmx.de", role = "aut"),
    person("Mark", "Heckmann", , "heckmann.mark@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-0736-7417")),
    person("ArData", role = "cph"),
    person("Frank", "Hangler", , "frank@plotandscatter.com", role = "ctb",
           comment = "function body_replace_all_text"),
    person("Liz", "Sander", , "lsander@civisanalytics.com", role = "ctb",
           comment = "several documentation fixes"),
    person("Anton", "Victorson", , "anton@victorson.se", role = "ctb",
           comment = "fixes xml structures"),
    person("Jon", "Calder", , "jonmcalder@gmail.com", role = "ctb",
           comment = "update vignettes"),
    person("John", "Harrold", , "john.m.harrold@gmail.com", role = "ctb",
           comment = "function annotate_base"),
    person("John", "Muschelli", , "muschellij2@gmail.com", role = "ctb",
           comment = "google doc compatibility"),
    person("Bill", "Denney", , "wdenney@humanpredictions.com", role = "ctb",
           comment = c(ORCID = "0000-0002-5759-428X", "function as.matrix.rpptx")),
    person("Nikolai", "Beck", , "beck.nikolai@gmail.com", role = "ctb",
           comment = "set speaker notes for .pptx documents"),
    person("Greg", "Leleu", , "gregoire.leleu@gmail.com", role = "ctb",
           comment = "fields functionality in ppt"),
    person("Majid", "Eismann", role = "ctb"),
    person("Wahiduzzaman", "Khan", role = "ctb",
           comment = "vectorization of remove_slide"),
    person("Hongyuan", "Jia", , "hongyuanjia@cqust.edu.cn", role = "ctb",
           comment = c(ORCID = "0000-0002-0075-8183")),
    person("Michael", "Stackhouse", , "mike.stackhouse@atorusresearch.com", role = "ctb")
  )
Description: Access and manipulate 'Microsoft Word', 'RTF' and 'Microsoft
    PowerPoint' documents from R.  The package focuses on tabular and
    graphical reporting from R; it also provides two functions that let
    users get document content into data objects. A set of functions lets
    add and remove images, tables and paragraphs of text in new or
    existing documents.  The package does not require any installation of
    Microsoft products to be able to write Microsoft files.
License: MIT + file LICENSE
URL: https://ardata-fr.github.io/officeverse/,
        https://davidgohel.github.io/officer/
BugReports: https://github.com/davidgohel/officer/issues
Imports: cli, dplyr, graphics, grDevices, openssl, R6, ragg, stats,
        tidyr, utils, uuid, xml2 (>= 1.1.0), zip (>= 2.1.0)
Suggests: devEMF, doconv (>= 0.3.0), gdtools, ggplot2, knitr, magick,
        rmarkdown, rsvg, mschart, testthat, withr
Encoding: UTF-8
Collate: 'core_properties.R' 'custom_properties.R' 'defunct.R'
        'dev-utils.R' 'docx_add.R' 'docx_comments.R' 'docx_cursor.R'
        'docx_embed_font.R' 'docx_part.R' 'docx_replace.R'
        'docx_section.R' 'docx_settings.R' 'docx_styles.R'
        'docx_utils_funs.R' 'empty_content.R' 'formatting_properties.R'
        'fortify_docx.R' 'fortify_pptx.R' 'knitr_utils.R' 'officer.R'
        'ooxml.R' 'ooxml_block_objects.R' 'ooxml_run_objects.R'
        'openxml_content_type.R' 'openxml_document.R' 'pack_folder.R'
        'ph_location.R' 'post-proc.R' 'ppt_class_dir_collection.R'
        'ppt_classes.R' 'ppt_notes.R' 'ppt_ph_dedupe_layout.R'
        'ppt_ph_manipulate.R' 'ppt_ph_rename_layout.R'
        'ppt_ph_with_methods.R' 'pptx_informations.R'
        'pptx_layout_helper.R' 'pptx_matrix.R' 'utils.R'
        'pptx_slide_manip.R' 'read_docx.R' 'docx_write.R'
        'read_docx_styles.R' 'read_pptx.R' 'read_xlsx.R'
        'relationship.R' 'rtf.R' 'shape_properties.R' 'shorcuts.R'
        'docx_append_context.R' 'utils-xml.R' 'deprecated.R' 'zzz.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-15 12:39:25 UTC; davidgohel
Author: David Gohel [aut, cre],
  Stefan Moog [aut],
  Mark Heckmann [aut] (ORCID: <https://orcid.org/0000-0002-0736-7417>),
  ArData [cph],
  Frank Hangler [ctb] (function body_replace_all_text),
  Liz Sander [ctb] (several documentation fixes),
  Anton Victorson [ctb] (fixes xml structures),
  Jon Calder [ctb] (update vignettes),
  John Harrold [ctb] (function annotate_base),
  John Muschelli [ctb] (google doc compatibility),
  Bill Denney [ctb] (ORCID: <https://orcid.org/0000-0002-5759-428X>,
    function as.matrix.rpptx),
  Nikolai Beck [ctb] (set speaker notes for .pptx documents),
  Greg Leleu [ctb] (fields functionality in ppt),
  Majid Eismann [ctb],
  Wahiduzzaman Khan [ctb] (vectorization of remove_slide),
  Hongyuan Jia [ctb] (ORCID: <https://orcid.org/0000-0002-0075-8183>),
  Michael Stackhouse [ctb]
Maintainer: David Gohel <david.gohel@ardata.fr>
Repository: CRAN
Date/Publication: 2026-05-15 14:10:02 UTC
Built: R 4.5.2; ; 2026-05-15 18:16:16 UTC; unix
