# Generated by roxygen2: do not edit by hand

S3method("[",app_properties)
S3method("[",core_properties)
S3method("[",custom_properties)
S3method("[<-",app_properties)
S3method("[<-",core_properties)
S3method("[<-",custom_properties)
S3method(as.character,fp_tab)
S3method(as.character,fp_tabs)
S3method(as.character,officer_cursor)
S3method(as.data.frame,external_img)
S3method(as.data.frame,fpar)
S3method(as.matrix,rpptx)
S3method(body_add,block_caption)
S3method(body_add,block_list)
S3method(body_add,block_list_items)
S3method(body_add,block_pour_docx)
S3method(body_add,block_section)
S3method(body_add,block_toc)
S3method(body_add,character)
S3method(body_add,data.frame)
S3method(body_add,external_img)
S3method(body_add,factor)
S3method(body_add,fpar)
S3method(body_add,gg)
S3method(body_add,numeric)
S3method(body_add,plot_instr)
S3method(body_add,run_columnbreak)
S3method(body_add,run_pagebreak)
S3method(dim,external_img)
S3method(format,fp_cell)
S3method(format,fp_par)
S3method(format,fp_text)
S3method(fortify_location,location_fullsize)
S3method(fortify_location,location_id)
S3method(fortify_location,location_label)
S3method(fortify_location,location_left)
S3method(fortify_location,location_manual)
S3method(fortify_location,location_right)
S3method(fortify_location,location_template)
S3method(fortify_location,location_type)
S3method(length,rdocx)
S3method(length,rpptx)
S3method(length,rxlsx)
S3method(ph_from_location,location_label)
S3method(ph_from_location,location_type)
S3method(print,block_caption)
S3method(print,block_list_items)
S3method(print,block_pour_docx)
S3method(print,block_section)
S3method(print,block_table)
S3method(print,block_toc)
S3method(print,fp_border)
S3method(print,fp_cell)
S3method(print,fp_par)
S3method(print,fp_text)
S3method(print,ftext)
S3method(print,layout_info)
S3method(print,rdocx)
S3method(print,rpptx)
S3method(print,rtf)
S3method(print,run_wordtext)
S3method(print,rxlsx)
S3method(print,sp_line)
S3method(print,sp_lineend)
S3method(print,unordered_list)
S3method(rtf_add,block_list)
S3method(rtf_add,block_section)
S3method(rtf_add,block_toc)
S3method(rtf_add,character)
S3method(rtf_add,double)
S3method(rtf_add,factor)
S3method(rtf_add,fpar)
S3method(rtf_add,gg)
S3method(rtf_add,plot_instr)
S3method(set_notes,block_list)
S3method(set_notes,character)
S3method(sheet_add_drawing,external_img)
S3method(sheet_add_drawing,gg)
S3method(sheet_write_data,block_list)
S3method(sheet_write_data,character)
S3method(sheet_write_data,data.frame)
S3method(sheet_write_data,default)
S3method(sheet_write_data,fpar)
S3method(to_html,block_list)
S3method(to_html,external_img)
S3method(to_html,fpar)
S3method(to_html,ftext)
S3method(to_html,hyperlink_ftext)
S3method(to_pml,block_list)
S3method(to_pml,block_list_items)
S3method(to_pml,block_table)
S3method(to_pml,external_img)
S3method(to_pml,fpar)
S3method(to_pml,ftext)
S3method(to_pml,hyperlink_ftext)
S3method(to_pml,sp_line)
S3method(to_pml,table_conditional_formatting)
S3method(to_pml,unordered_list)
S3method(to_rtf,block_list)
S3method(to_rtf,block_section)
S3method(to_rtf,block_toc)
S3method(to_rtf,default)
S3method(to_rtf,external_img)
S3method(to_rtf,floating_external_img)
S3method(to_rtf,fpar)
S3method(to_rtf,ftext)
S3method(to_rtf,hyperlink_ftext)
S3method(to_rtf,page_mar)
S3method(to_rtf,page_size)
S3method(to_rtf,prop_section)
S3method(to_rtf,run_autonum)
S3method(to_rtf,run_columnbreak)
S3method(to_rtf,run_linebreak)
S3method(to_rtf,run_pagebreak)
S3method(to_rtf,run_reference)
S3method(to_rtf,run_tab)
S3method(to_rtf,run_word_field)
S3method(to_rtf,section_columns)
S3method(to_wml,block_caption)
S3method(to_wml,block_gg)
S3method(to_wml,block_list)
S3method(to_wml,block_list_items)
S3method(to_wml,block_pour_docx)
S3method(to_wml,block_section)
S3method(to_wml,block_table)
S3method(to_wml,block_toc)
S3method(to_wml,external_img)
S3method(to_wml,floating_external_img)
S3method(to_wml,fp_cell)
S3method(to_wml,fp_par)
S3method(to_wml,fp_tab)
S3method(to_wml,fp_tabs)
S3method(to_wml,fp_text)
S3method(to_wml,fpar)
S3method(to_wml,ftext)
S3method(to_wml,hyperlink_ftext)
S3method(to_wml,page_mar)
S3method(to_wml,page_size)
S3method(to_wml,prop_section)
S3method(to_wml,prop_table)
S3method(to_wml,run_autonum)
S3method(to_wml,run_bookmark)
S3method(to_wml,run_columnbreak)
S3method(to_wml,run_comment)
S3method(to_wml,run_footnote)
S3method(to_wml,run_footnoteref)
S3method(to_wml,run_linebreak)
S3method(to_wml,run_pagebreak)
S3method(to_wml,run_reference)
S3method(to_wml,run_tab)
S3method(to_wml,run_word_field)
S3method(to_wml,run_wordtext)
S3method(to_wml,section_columns)
S3method(to_wml,table_colwidths)
S3method(to_wml,table_conditional_formatting)
S3method(to_wml,table_layout)
S3method(to_wml,table_stylenames)
S3method(to_wml,table_width)
S3method(update,fp_border)
S3method(update,fp_cell)
S3method(update,fp_par)
S3method(update,fp_text)
S3method(update,fpar)
S3method(update,sp_line)
S3method(update,sp_lineend)
export("layout_rename_ph_labels<-")
export("slide_visible<-")
export(add_sheet)
export(add_slide)
export(annotate_base)
export(as_base64)
export(base64_to_image)
export(block_caption)
export(block_gg)
export(block_list)
export(block_list_items)
export(block_pour_docx)
export(block_section)
export(block_table)
export(block_toc)
export(body_add)
export(body_add_blocks)
export(body_add_break)
export(body_add_caption)
export(body_add_docx)
export(body_add_fpar)
export(body_add_gg)
export(body_add_img)
export(body_add_list)
export(body_add_par)
export(body_add_plot)
export(body_add_table)
export(body_add_toc)
export(body_add_xml)
export(body_append_start_context)
export(body_append_stop_context)
export(body_bookmark)
export(body_comment)
export(body_end_block_section)
export(body_end_section_columns)
export(body_end_section_columns_landscape)
export(body_end_section_continuous)
export(body_end_section_landscape)
export(body_end_section_portrait)
export(body_import_docx)
export(body_remove)
export(body_replace_all_text)
export(body_replace_gg_at_bkm)
export(body_replace_img_at_bkm)
export(body_replace_plot_at_bkm)
export(body_replace_text_at_bkm)
export(body_set_default_section)
export(change_styles)
export(color_scheme)
export(cursor_backward)
export(cursor_begin)
export(cursor_bookmark)
export(cursor_end)
export(cursor_forward)
export(cursor_reach)
export(cursor_reach_index)
export(cursor_reach_test)
export(doc_properties)
export(docx_body_relationship)
export(docx_body_xml)
export(docx_bookmarks)
export(docx_comments)
export(docx_current_block_xml)
export(docx_dim)
export(docx_embed_font)
export(docx_reference_img)
export(docx_set_character_style)
export(docx_set_paragraph_style)
export(docx_set_settings)
export(docx_show_chunk)
export(docx_summary)
export(empty_content)
export(external_img)
export(floating_external_img)
export(footers_replace_all_text)
export(footers_replace_img_at_bkm)
export(footers_replace_text_at_bkm)
export(fortify_location)
export(fp_border)
export(fp_cell)
export(fp_par)
export(fp_par_lite)
export(fp_tab)
export(fp_tabs)
export(fp_text)
export(fp_text_lite)
export(fpar)
export(from_base64)
export(ftext)
export(get_reference_value)
export(headers_replace_all_text)
export(headers_replace_img_at_bkm)
export(headers_replace_text_at_bkm)
export(hyperlink_ftext)
export(image_to_base64)
export(layout_dedupe_ph_labels)
export(layout_default)
export(layout_properties)
export(layout_rename_ph_labels)
export(layout_summary)
export(list_item)
export(media_extract)
export(move_slide)
export(notes_location_label)
export(notes_location_type)
export(officer_url_encode)
export(on_slide)
export(ooxml_chart_uris)
export(open_file)
export(opts_current_table)
export(pack_folder)
export(page_mar)
export(page_size)
export(ph_hyperlink)
export(ph_location)
export(ph_location_fullsize)
export(ph_location_id)
export(ph_location_label)
export(ph_location_left)
export(ph_location_right)
export(ph_location_template)
export(ph_location_type)
export(ph_remove)
export(ph_slidelink)
export(ph_with)
export(ph_with.Date)
export(ph_with.block_list)
export(ph_with.block_list_items)
export(ph_with.character)
export(ph_with.data.frame)
export(ph_with.empty_content)
export(ph_with.external_img)
export(ph_with.factor)
export(ph_with.fpar)
export(ph_with.gg)
export(ph_with.logical)
export(ph_with.numeric)
export(ph_with.plot_instr)
export(ph_with.unordered_list)
export(ph_with.xml_document)
export(phs_with)
export(plot_in_png)
export(plot_instr)
export(plot_layout_properties)
export(pptx_summary)
export(prop_section)
export(prop_table)
export(read_docx)
export(read_pptx)
export(read_xlsx)
export(remove_slide)
export(resolve_location)
export(rtf_add)
export(rtf_doc)
export(rtf_set_paragraph_style)
export(rtf_styles_info)
export(run_autonum)
export(run_bookmark)
export(run_columnbreak)
export(run_comment)
export(run_footnote)
export(run_footnoteref)
export(run_linebreak)
export(run_pagebreak)
export(run_reference)
export(run_tab)
export(run_word_field)
export(run_wordtext)
export(sanitize_images)
export(section_columns)
export(set_autonum_bookmark)
export(set_doc_properties)
export(set_notes)
export(shape_properties_tags)
export(sheet_add_drawing)
export(sheet_names)
export(sheet_remove)
export(sheet_select)
export(sheet_write_data)
export(shortcuts)
export(slide_size)
export(slide_summary)
export(slide_visible)
export(solid_fill)
export(sp_line)
export(sp_lineend)
export(str_encode_to_rtf)
export(styles_info)
export(table_colwidths)
export(table_conditional_formatting)
export(table_layout)
export(table_stylenames)
export(table_width)
export(to_html)
export(to_pml)
export(to_rtf)
export(to_wml)
export(unordered_list)
export(unpack_folder)
export(uuid_generate)
export(wml_link_images)
export(write_elements_to_context)
export(xlsx_drawing)
export(xlsx_styles)
import(graphics)
importFrom(R6,R6Class)
importFrom(cli,cli_abort)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,consecutive_id)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,relocate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,syms)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,rgb)
importFrom(graphics,box)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(openssl,base64_decode)
importFrom(openssl,base64_encode)
importFrom(openssl,sha1)
importFrom(ragg,agg_png)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(tidyr,complete)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(uuid,UUIDgenerate)
importFrom(xml2,"xml_attr<-")
importFrom(xml2,"xml_name<-")
importFrom(xml2,"xml_text<-")
importFrom(xml2,as_list)
importFrom(xml2,as_xml_document)
importFrom(xml2,read_xml)
importFrom(xml2,write_xml)
importFrom(xml2,xml_add_child)
importFrom(xml2,xml_add_parent)
importFrom(xml2,xml_add_sibling)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_attrs)
importFrom(xml2,xml_child)
importFrom(xml2,xml_children)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_has_attr)
importFrom(xml2,xml_length)
importFrom(xml2,xml_missing)
importFrom(xml2,xml_name)
importFrom(xml2,xml_ns)
importFrom(xml2,xml_parent)
importFrom(xml2,xml_path)
importFrom(xml2,xml_remove)
importFrom(xml2,xml_replace)
importFrom(xml2,xml_set_attr)
importFrom(xml2,xml_set_attrs)
importFrom(xml2,xml_set_text)
importFrom(xml2,xml_text)
importFrom(zip,unzip)
importFrom(zip,zip)
