#' @name officer-defunct
#' @title Defunct Functions in Package officer
#' @export
#' @rdname officer-defunct
#' @details `docx_reference_img()` is not usefull anymore.
#' @param ... unused arguments
#' @keywords internal
docx_reference_img <- function(...) {
  .Defunct(msg = "The `docx_reference_img()` function is no longer useful.")
}
