% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_xlsx.R
\name{sheet_select}
\alias{sheet_select}
\title{Select sheet}
\usage{
sheet_select(x, sheet)
}
\arguments{
\item{x}{rxlsx object}

\item{sheet}{sheet name}
}
\description{
Set a particular sheet selected when workbook will be
edited.
}
\examples{
my_ws <- read_xlsx()
my_pres <- add_sheet(my_ws, label = "new sheet")
my_pres <- sheet_select(my_ws, sheet = "new sheet")
print(my_ws, target = tempfile(fileext = ".xlsx") )
}
