% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppt_ph_manipulate.R
\name{ph_slidelink}
\alias{ph_slidelink}
\title{Slide link to a placeholder}
\usage{
ph_slidelink(
  x,
  type = "body",
  id = 1,
  id_chr = NULL,
  ph_label = NULL,
  slide_index
)
}
\arguments{
\item{x}{an rpptx object}

\item{type}{placeholder type}

\item{id}{placeholder index (integer) for a duplicated type. This is to be used when a placeholder
type is not unique in the layout of the current slide, e.g. two placeholders with type 'body'. To
add onto the first, use \code{id = 1} and \code{id = 2} for the second one.
Values can be read from \code{\link[=slide_summary]{slide_summary()}}.}

\item{id_chr}{deprecated.}

\item{ph_label}{label associated to the placeholder. Use column
\code{ph_label} of result returned by \code{\link[=slide_summary]{slide_summary()}}.
If used, \code{type} and \code{id} are ignored.}

\item{slide_index}{slide index to reach}
}
\description{
Add slide link to a placeholder in the current slide.
}
\examples{
fileout <- tempfile(fileext = ".pptx")
loc_title <- ph_location_type(type = "title")
doc <- read_pptx()
doc <- add_slide(doc, "Title and Content")
doc <- ph_with(x = doc, "Un titre 1", location = loc_title)
doc <- add_slide(doc, "Title and Content")
doc <- ph_with(x = doc, "Un titre 2", location = loc_title)
doc <- on_slide(doc, 1)
slide_summary(doc) # read column ph_label here
doc <- ph_slidelink(x = doc, ph_label = "Title 1", slide_index = 2)

print(doc, target = fileout)
}
\seealso{
\code{\link[=ph_with]{ph_with()}}

Other functions for placeholders manipulation: 
\code{\link{ph_hyperlink}()},
\code{\link{ph_remove}()}
}
\concept{functions for placeholders manipulation}
