% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ooxml_block_objects.R
\name{block_list_items}
\alias{block_list_items}
\title{List of items for Word and PowerPoint}
\usage{
block_list_items(..., list_type = "bullet")
}
\arguments{
\item{...}{\code{\link[=list_item]{list_item()}} objects}

\item{list_type}{\code{"bullet"} for an unordered list or \code{"decimal"}
for a numbered list}
}
\description{
Create a bullet or numbered list from \code{\link[=list_item]{list_item()}}
elements. Supports rich text via \code{\link[=fpar]{fpar()}} and multi-level nesting.
Works in both Word and PowerPoint documents.
}
\examples{
items <- block_list_items(
  list_item(fpar(
    ftext("Item 1", fp_text(color = "red"))
  ), level = 1),
  list_item(fpar("Sub-item"), level = 2),
  list_item(fpar("Item 2"), level = 1),
  list_type = "bullet"
)
items
}
\seealso{
\code{\link[=list_item]{list_item()}}, \code{\link[=body_add]{body_add()}}, \code{\link[=ph_with]{ph_with()}}

Other block functions for reporting:
\code{\link[=block_caption]{block_caption()}},
\code{\link[=block_gg]{block_gg()}},
\code{\link[=block_list]{block_list()}},
\code{\link[=block_pour_docx]{block_pour_docx()}},
\code{\link[=block_section]{block_section()}},
\code{\link[=block_table]{block_table()}},
\code{\link[=block_toc]{block_toc()}},
\code{\link[=fpar]{fpar()}},
\code{\link[=list_item]{list_item()}},
\code{\link[=plot_instr]{plot_instr()}},
\code{\link[=unordered_list]{unordered_list()}}
}
\concept{block functions for reporting}
