% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ooxml_run_objects.R
\name{run_footnoteref}
\alias{run_footnoteref}
\title{Word footnote reference}
\usage{
run_footnoteref(prop = NULL)
}
\arguments{
\item{prop}{formatting text properties returned by
\code{\link[=fp_text_lite]{fp_text_lite()}} or \code{\link[=fp_text]{fp_text()}}. It also can be NULL in
which case, no formatting is defined (the default is applied).}
}
\description{
Wraps a footnote reference in an object that can then be inserted
as a run/chunk with \code{\link[=fpar]{fpar()}} or within an R Markdown document.
}
\examples{
run_footnoteref()
to_wml(run_footnoteref())
}
\seealso{
Other run functions for reporting:
\code{\link[=external_img]{external_img()}},
\code{\link[=floating_external_img]{floating_external_img()}},
\code{\link[=ftext]{ftext()}},
\code{\link[=hyperlink_ftext]{hyperlink_ftext()}},
\code{\link[=run_autonum]{run_autonum()}},
\code{\link[=run_bookmark]{run_bookmark()}},
\code{\link[=run_columnbreak]{run_columnbreak()}},
\code{\link[=run_comment]{run_comment()}},
\code{\link[=run_footnote]{run_footnote()}},
\code{\link[=run_linebreak]{run_linebreak()}},
\code{\link[=run_pagebreak]{run_pagebreak()}},
\code{\link[=run_reference]{run_reference()}},
\code{\link[=run_tab]{run_tab()}},
\code{\link[=run_word_field]{run_word_field()}},
\code{\link[=run_wordtext]{run_wordtext()}}
}
\concept{run functions for reporting}
