% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_xlsx.R
\name{sheet_remove}
\alias{sheet_remove}
\title{Remove a sheet}
\usage{
sheet_remove(x, sheet)
}
\arguments{
\item{x}{rxlsx object}

\item{sheet}{name of the sheet to remove}
}
\value{
the rxlsx object (invisibly)
}
\description{
Remove a sheet from an xlsx workbook, deleting the
worksheet XML, its relationship file and the content-type override.
}
\examples{
wb <- read_xlsx()
wb <- add_sheet(wb, "kept")
default_name <- sheet_names(wb)[1]
wb <- sheet_remove(wb, sheet = default_name)
print(wb, target = tempfile(fileext = ".xlsx"))
}
